//this file is part of eMule
//Copyright (C)2002 Merkur ( merkur-@users.sourceforge.net / http://emule.sf.net )
//
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either
//version 2 of the License, or (at your option) any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#include "StdAfx.h"
#include "UpDownClient.h"
#include "opcodes.h"
#include "packets.h"
#include "emule.h"
#include "uploadqueue.h"

//	members of CUpDownClient
//	which are mainly used for uploading functions 

uint32 CUpDownClient::GetScore(bool isdownloading){
	//TODO: complete this (friends, uploadspeed, emuleuser etc etc)
	isbotuser = strstr(username,"finder") && strstr(username,"com");
	isfriend = false;
	isemuleclient = false;
	filepriority = 10; // standart	
	// calculate score, based on waitingtime and other factors
	float basevalue;
	if (!isdownloading)
		basevalue = (float)(::GetTickCount()-starttime)/1000;
	else{
		// we dont want one client to download forever
		// the first 15 min downloadtime counts as 15 min waitingtime and you get a 15 min bonus while you are in the first 15 min :)
		// (to avoid 20 sec downloads) after this the score won't raise anymore 
		basevalue = (float)(upstarttime-starttime);
		basevalue += (float)(::GetTickCount() - upstarttime > 900000)? 900000:1800000;
		basevalue /= 1000;
	}	
	if (isbotuser)
		basevalue *= 0.8f;
	if (haslowid)
		basevalue *= 0.7f;
	//if (isuploading)
	//	basevalue *= 1.8f;
	if (isfriend)
		basevalue *= 2.0f;
	if (isemuleclient)
		basevalue *= 1.3f;
	basevalue *= ((float)filepriority/10);
	return (uint32)basevalue;
}

bool CUpDownClient::CreateNextBlockPackage(){
	// check if we should kick this client
	if (theApp.uploadqueue->CheckForTimeOver(this)){
		// back on the waitqueue
		SetWaitStartTime();
		theApp.uploadqueue->RemoveFromUploadQueue(this);
		theApp.uploadqueue->AddClientToQueue(this);
		return false;
	}
	if (cur_block){
		cur_block->Close();
		delete cur_block;
		cur_block = 0;
	}
	if (blockreqlist.IsEmpty())
		return false;
	CMemFile* blockpack = new CMemFile();
	FILE* file = 0;
	char* buffer = 0;
	try{
		while (!blockreqlist.IsEmpty()){
			Requested_Block_Struct* currentblock = blockreqlist.GetHead();
			CKnownFile* srcfile = theApp.sharedfiles->GetFileByID(currentblock->FileID);
			if (!srcfile)
				throw "requested file not found";

			char* fullname;
			if (!srcfile->IsPartFile()){
				fullname = new char[strlen(srcfile->GetPath())+strlen(srcfile->GetFileName())+10];
				sprintf(fullname,"%s\\%s",srcfile->GetPath(),srcfile->GetFileName());
			}
			else{
				fullname = strdup(((CPartFile*)srcfile)->GetFullName());
				fullname[strlen(fullname)-4] = 0;
			}
			file = fopen(fullname,"rbS");
			delete[] fullname;
			if (!file)
				throw "failed to open requested file";

			fseek(file,currentblock->StartOffset,0);
			uint32 togo;
			if (currentblock->StartOffset > currentblock->EndOffset){
				togo = currentblock->EndOffset + (srcfile->GetFileSize() - currentblock->StartOffset);
			}
			else{
				togo = currentblock->EndOffset - currentblock->StartOffset;
				if (srcfile->IsPartFile() && !((CPartFile*)srcfile)->IsComplete(currentblock->StartOffset,currentblock->EndOffset-1))
					throw "asked for incomplete block ";
			}
			uint32 packetsize;
			if (togo > 10240) 
				packetsize = togo/(uint32)(togo/10240);
			else
				packetsize = togo;

			while (togo){
				if (togo < packetsize*2)
					packetsize = togo;
				togo -= packetsize;
				buffer = new char[packetsize+30];
				if (uint32 done = fread(&buffer[30],1,packetsize,file) != packetsize){
					fseek(file,0,0);
					fread(&buffer[30+done],packetsize-done,1,file);
				}
				if (ferror(file))
					throw "unexpected error while reading requested file";

				Packet* packet = new Packet(OP_SENDINGPART,0);
				packet->size = packetsize+24; // + 24 fileid + offsets (headersize is added at getheader)
				memcpy(buffer,packet->GetHeader(),6);
				delete packet;
				memcpy(&buffer[6],srcfile->GetFileHash(),16);
				uint32 statpos = (currentblock->EndOffset - togo) - packetsize;
				memcpy(&buffer[22],&statpos,4);
				uint32 endpos = (currentblock->EndOffset - togo);
				memcpy(&buffer[26],&endpos,4);
				blockpack->Write(buffer,packetsize+30);
				delete[] buffer;
				buffer = 0;
			}
			fclose(file);
			file = 0;
			memcpy(reqfileid,currentblock->FileID,16);
			doneblocklist.AddHead(blockreqlist.RemoveHead());
		}
		blockpack->SeekToBegin();
		cur_block = blockpack;
	}
	catch(char* error){
		theApp.emuledlg->AddLogLine(false,"Client '%s' caused error while creating package (%s) - disconnecting client",GetUserName(),error);
		socket->Disconnect();
		if (file)
			fclose(file);
		delete blockpack;
		if (buffer)
			delete buffer;
		return false;
	}
	//theApp.emuledlg->AddLogLine(false,"Debug: Packet done. Size: %i",blockpack->GetLength());
	return true;
}

void CUpDownClient::AddReqBlock(Requested_Block_Struct* reqblock){

	for (POSITION pos = doneblocklist.GetHeadPosition();pos != 0;doneblocklist.GetNext(pos)){
		if (reqblock->StartOffset == doneblocklist.GetAt(pos)->StartOffset && reqblock->EndOffset == doneblocklist.GetAt(pos)->EndOffset){
			delete reqblock;
			return;
		}
	}
	for (POSITION pos = blockreqlist.GetHeadPosition();pos != 0;blockreqlist.GetNext(pos)){
		if (reqblock->StartOffset == blockreqlist.GetAt(pos)->StartOffset && reqblock->EndOffset == blockreqlist.GetAt(pos)->EndOffset){
			delete reqblock;
			return;
		}
	}
	blockreqlist.AddTail(reqblock);

}

void CUpDownClient::SetUpStartTime(uint32 time){
	if (time)
		upstarttime = time;
	else
		upstarttime = ::GetTickCount();
}

void CUpDownClient::SetWaitStartTime(uint32 time){
	if (time)
		starttime = time;
	else
		starttime = ::GetTickCount();
}

uint32 CUpDownClient::SendBlockData(uint32 maxammount){
	dataratetmp += dataratems;
	count++;
	if (count == 100){
		datarate = dataratetmp/10;
		count = 0;
		dataratetmp = 0;
		theApp.emuledlg->transferwnd.uploadlistctrl.RefreshClient(this);
	}
	dataratems = 0;
	if (!socket->IsBusy()){
		if (cur_block){
			char* sendbuffer = new char[maxammount];
			uint32 rdone = 0;
			rdone = cur_block->Read(sendbuffer,maxammount);
			socket->Send(sendbuffer,rdone);
			if (maxammount-rdone)
				CreateNextBlockPackage();
			delete[] sendbuffer;
			transfered += rdone;
			dataratems += rdone;
			return rdone;
		}
		else{
			CreateNextBlockPackage();
			return 0;
		}
	}
	return 0;
}